@@ -145,7 +145,7 @@ int main(int argc, char* argv[])
 
 #endif
      
-    if (DSP == 0) {
+    if (!DSP) {
         std::cerr << "Unable to allocate Faust DSP object" << std::endl;
         exit(1);
     }
